@@ -29,21 +29,24 @@ KIND_MAP = {kind: _hidpp10.DEVICE_KIND[str(kind)] for kind in _hidpp20.DEVICE_KI
 
 
 class Device:
-
+    instances = []
     read_register = _hidpp10.read_register
     write_register = _hidpp10.write_register
 
     def __init__(self, receiver, number, link_notification=None, info=None, path=None, handle=None):
         assert receiver or info
+        Device.instances.append(self)
         self.receiver = receiver
         self.may_unpair = False
         self.isDevice = True  # some devices act as receiver so we need a property to distinguish them
         self.path = path
         self.handle = handle
         self.product_id = None
+        self.hidpp_short = info.hidpp_short if info else None
+        self.hidpp_long = info.hidpp_long if info else None
 
         if receiver:
-            assert number > 0 and number <= receiver.max_devices
+            assert number > 0 and number <= 15  # some receivers have devices past their max # of devices
         self.number = number  # will be None at this point for directly connected devices
         # 'device active' flag; requires manual management.
         self.online = None
@@ -113,18 +116,19 @@ class Device:
                 # assert link_notification.address == (0x04 if unifying else 0x03)
                 kind = ord(link_notification.data[0:1]) & 0x0F
                 # get 27Mhz wpid and set kind based on index
-                if receiver.ex100_27mhz_wpid_fix:  # 27 Mhz receiver
+                if receiver.receiver_kind == '27Mhz':  # 27 Mhz receiver
                     self.wpid = '00' + _strhex(link_notification.data[2:3])
                     kind = self.get_kind_from_index(number, receiver)
                 self._kind = _hidpp10.DEVICE_KIND[kind]
             else:
-                # Not a notification, force a reading of the wpid
+                # Not a notification, force a reading of pairing information
                 self.online = True
                 self.update_pairing_information()
+                self.update_extended_pairing_information()
+                if not self.wpid and not self._serial:  # if neither then the device almost certainly wasn't found
+                    raise _base.NoSuchDevice(number=number, receiver=receiver, error='no wpid or serial')
 
-            # the wpid is necessary to properly identify wireless link on/off
-            # notifications also it gets set to None on this object when the
-            # device is unpaired
+            # the wpid is set to None on this object when the device is unpaired
             assert self.wpid is not None, 'failed to read wpid: device %d of %s' % (number, receiver)
 
             self.descriptor = _descriptors.get_wpid(self.wpid)
@@ -158,10 +162,21 @@ class Device:
             # may be a 2.0 device; if not, it will fix itself later
             self.features = _hidpp20.FeaturesArray(self)
 
+    @classmethod
+    def find(self, serial):
+        assert serial, 'need serial number or unit ID to find a device'
+        result = None
+        for device in Device.instances:
+            if device.online and (device.unitId == serial or device.serial == serial):
+                result = device
+        return result
+
     @property
     def protocol(self):
         if not self._protocol and self.online:
-            self._protocol = _base.ping(self.handle or self.receiver.handle, self.number, long_message=self.bluetooth)
+            self._protocol = _base.ping(
+                self.handle or self.receiver.handle, self.number, long_message=self.bluetooth or self.hidpp_short is False
+            )
             # if the ping failed, the peripheral is (almost) certainly offline
             self.online = self._protocol is not None
 
@@ -217,7 +232,7 @@ class Device:
         return self._tid_map
 
     def update_pairing_information(self):
-        if self.receiver:
+        if self.receiver and (not self.wpid or self._kind is None or self._polling_rate is None):
             wpid, kind, polling_rate = self.receiver.device_pairing_information(self.number)
             if not self.wpid:
                 self.wpid = wpid
@@ -335,12 +350,35 @@ class Device:
                     self._feature_settings_checked = _check_feature_settings(self, self._settings)
         return self._settings
 
+    def set_configuration(self, configuration, no_reply=False):
+        if self.online and self.protocol >= 2.0:
+            _hidpp20.config_change(self, configuration, no_reply=no_reply)
+
+    def reset(self, no_reply=False):
+        self.set_configuration(0, no_reply)
+
     @property
     def persister(self):
         if not self._persister:
             self._persister = _configuration.persister(self)
         return self._persister
 
+    def battery(self):  # None  or  level, next, status, voltage
+        if self.protocol < 2.0:
+            return _hidpp10.get_battery(self)
+        else:
+            battery_feature = self.persister.get('_battery', None) if self.persister else None
+            if battery_feature != 0:
+                result = _hidpp20.get_battery(self, battery_feature)
+                try:
+                    feature, level, next, status, voltage = result
+                    if self.persister and battery_feature is None:
+                        self.persister['_battery'] = feature
+                    return level, next, status, voltage
+                except Exception:
+                    if self.persister and battery_feature is None:
+                        self.persister['_battery'] = result
+
     def enable_connection_notifications(self, enable=True):
         """Enable or disable device (dis)connection notifications on this
         receiver."""
@@ -403,7 +441,7 @@ class Device:
                 request_id,
                 *params,
                 no_reply=no_reply,
-                long_message=self.bluetooth or self.protocol >= 2.0,
+                long_message=self.bluetooth or self.hidpp_short is False or self.protocol >= 2.0,
                 protocol=self.protocol
             )
 
@@ -426,7 +464,7 @@ class Device:
     __int__ = __index__
 
     def __eq__(self, other):
-        return other is not None and self.kind == other.kind and self.wpid == other.wpid
+        return other is not None and self._kind == other._kind and self.wpid == other.wpid
 
     def __ne__(self, other):
         return other is None or self.kind != other.kind or self.wpid != other.wpid
@@ -467,6 +505,8 @@ class Device:
 
     def close(self):
         handle, self.handle = self.handle, None
+        if self in Device.instances:
+            Device.instances.remove(self)
         return (handle and _base.close(handle))
 
     def __del__(self):
