@@ -42,6 +42,7 @@ from .hidpp20 import FEATURE as _F
 from .special_keys import CONTROL as _CONTROL
 
 import gi  # isort:skip
+
 gi.require_version('Gdk', '3.0')  # isort:skip
 from gi.repository import Gdk, GLib  # NOQA: E402 # isort:skip
 
@@ -337,23 +338,49 @@ def simulate_scroll(dx, dy):
     _log.warn('no way to simulate scrolling')
 
 
+def thumb_wheel_up(f, r, d, a):
+    global thumb_wheel_displacement
+    if f != _F.THUMB_WHEEL or r != 0:
+        return False
+    if a is None:
+        return signed(d[0:2]) < 0 and signed(d[0:2])
+    elif thumb_wheel_displacement <= -a:
+        thumb_wheel_displacement += a
+        return 1
+    else:
+        return False
+
+
+def thumb_wheel_down(f, r, d, a):
+    global thumb_wheel_displacement
+    if f != _F.THUMB_WHEEL or r != 0:
+        return False
+    if a is None:
+        return signed(d[0:2]) > 0 and signed(d[0:2])
+    elif thumb_wheel_displacement >= a:
+        thumb_wheel_displacement -= a
+        return 1
+    else:
+        return False
+
+
 TESTS = {
-    'crown_right': lambda f, r, d: f == _F.CROWN and r == 0 and d[1] < 128 and d[1],
-    'crown_left': lambda f, r, d: f == _F.CROWN and r == 0 and d[1] >= 128 and 256 - d[1],
-    'crown_right_ratchet': lambda f, r, d: f == _F.CROWN and r == 0 and d[2] < 128 and d[2],
-    'crown_left_ratchet': lambda f, r, d: f == _F.CROWN and r == 0 and d[2] >= 128 and 256 - d[2],
-    'crown_tap': lambda f, r, d: f == _F.CROWN and r == 0 and d[5] == 0x01 and d[5],
-    'crown_start_press': lambda f, r, d: f == _F.CROWN and r == 0 and d[6] == 0x01 and d[6],
-    'crown_end_press': lambda f, r, d: f == _F.CROWN and r == 0 and d[6] == 0x05 and d[6],
-    'crown_pressed': lambda f, r, d: f == _F.CROWN and r == 0 and d[6] >= 0x01 and d[6] <= 0x04 and d[6],
-    'thumb_wheel_up': lambda f, r, d: f == _F.THUMB_WHEEL and r == 0 and signed(d[0:2]) < 0 and signed(d[0:2]),
-    'thumb_wheel_down': lambda f, r, d: f == _F.THUMB_WHEEL and r == 0 and signed(d[0:2]) > 0 and signed(d[0:2]),
-    'lowres_wheel_up': lambda f, r, d: f == _F.LOWRES_WHEEL and r == 0 and signed(d[0:1]) > 0 and signed(d[0:1]),
-    'lowres_wheel_down': lambda f, r, d: f == _F.LOWRES_WHEEL and r == 0 and signed(d[0:1]) < 0 and signed(d[0:1]),
-    'hires_wheel_up': lambda f, r, d: f == _F.HIRES_WHEEL and r == 0 and signed(d[1:3]) > 0 and signed(d[1:3]),
-    'hires_wheel_down': lambda f, r, d: f == _F.HIRES_WHEEL and r == 0 and signed(d[1:3]) < 0 and signed(d[1:3]),
-    'False': lambda f, r, d: False,
-    'True': lambda f, r, d: True,
+    'crown_right': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[1] < 128 and d[1], False],
+    'crown_left': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[1] >= 128 and 256 - d[1], False],
+    'crown_right_ratchet': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[2] < 128 and d[2], False],
+    'crown_left_ratchet': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[2] >= 128 and 256 - d[2], False],
+    'crown_tap': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[5] == 0x01 and d[5], False],
+    'crown_start_press': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[6] == 0x01 and d[6], False],
+    'crown_end_press': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[6] == 0x05 and d[6], False],
+    'crown_pressed': [lambda f, r, d, a: f == _F.CROWN and r == 0 and d[6] >= 0x01 and d[6] <= 0x04 and d[6], False],
+    'thumb_wheel_up': [thumb_wheel_up, True],
+    'thumb_wheel_down': [thumb_wheel_down, True],
+    'lowres_wheel_up': [lambda f, r, d, a: f == _F.LOWRES_WHEEL and r == 0 and signed(d[0:1]) > 0 and signed(d[0:1]), False],
+    'lowres_wheel_down': [lambda f, r, d, a: f == _F.LOWRES_WHEEL and r == 0 and signed(d[0:1]) < 0 and signed(d[0:1]), False],
+    'hires_wheel_up': [lambda f, r, d, a: f == _F.HIRES_WHEEL and r == 0 and signed(d[1:3]) > 0 and signed(d[1:3]), False],
+    'hires_wheel_down': [lambda f, r, d, a: f == _F.HIRES_WHEEL and r == 0 and signed(d[1:3]) < 0 and signed(d[1:3]), False],
+    'False': [lambda f, r, d, a: False, False],
+    'True': [lambda f, r, d, a: True, False],
 }
 
 MOUSE_GESTURE_TESTS = {
@@ -368,6 +395,7 @@ COMPONENTS = {}
 
 
 class RuleComponent:
+
     def compile(self, c):
         if isinstance(c, RuleComponent):
             return c
@@ -380,6 +408,7 @@ class RuleComponent:
 
 
 class Rule(RuleComponent):
+
     def __init__(self, args, source=None, warn=True):
         self.components = [self.compile(a) for a in args]
         self.source = source
@@ -398,11 +427,16 @@ class Rule(RuleComponent):
                 return result
         return result
 
+    def once(self, feature, notification, device, status, last_result):
+        self.evaluate(feature, notification, device, status, last_result)
+        return False
+
     def data(self):
         return {'Rule': [c.data() for c in self.components]}
 
 
 class Condition(RuleComponent):
+
     def __init__(self, *args):
         pass
 
@@ -414,6 +448,7 @@ class Condition(RuleComponent):
 
 
 class Not(Condition):
+
     def __init__(self, op, warn=True):
         if isinstance(op, list) and len(op) == 1:
             op = op[0]
@@ -432,6 +467,7 @@ class Not(Condition):
 
 
 class Or(Condition):
+
     def __init__(self, args, warn=True):
         self.components = [self.compile(a) for a in args]
 
@@ -453,6 +489,7 @@ class Or(Condition):
 
 
 class And(Condition):
+
     def __init__(self, args, warn=True):
         self.components = [self.compile(a) for a in args]
 
@@ -506,6 +543,7 @@ def x11_pointer_prog():
 
 
 class Process(Condition):
+
     def __init__(self, process, warn=True):
         self.process = process
         if wayland or not x11_setup():
@@ -531,6 +569,7 @@ class Process(Condition):
 
 
 class MouseProcess(Condition):
+
     def __init__(self, process, warn=True):
         self.process = process
         if wayland or not x11_setup():
@@ -556,6 +595,7 @@ class MouseProcess(Condition):
 
 
 class Feature(Condition):
+
     def __init__(self, feature, warn=True):
         if not (isinstance(feature, str) and feature in _F):
             if warn:
@@ -574,6 +614,7 @@ class Feature(Condition):
 
 
 class Report(Condition):
+
     def __init__(self, report, warn=True):
         if not (isinstance(report, int)):
             if warn:
@@ -594,6 +635,7 @@ class Report(Condition):
 
 # Setting(device, setting, [key], value...)
 class Setting(Condition):
+
     def __init__(self, args, warn=True):
         if not (isinstance(args, list) and len(args) > 2):
             if warn:
@@ -639,6 +681,7 @@ MODIFIER_MASK = MODIFIERS['Shift'] + MODIFIERS['Control'] + MODIFIERS['Alt'] + M
 
 
 class Modifiers(Condition):
+
     def __init__(self, modifiers, warn=True):
         modifiers = [modifiers] if isinstance(modifiers, str) else modifiers
         self.desired = 0
@@ -721,6 +764,7 @@ def bit_test(start, end, bits):
 
 
 def range_test(start, end, min, max):
+
     def range_test_helper(f, r, d):
         value = int.from_bytes(d[start:end], byteorder='big', signed=True)
         return min <= value <= max and (value if value else True)
@@ -729,25 +773,32 @@ def range_test(start, end, min, max):
 
 
 class Test(Condition):
+
     def __init__(self, test, warn=True):
-        self.test = test
+        self.test = ''
+        self.parameter = None
         if isinstance(test, str):
-            if test in MOUSE_GESTURE_TESTS:
-                if warn:
-                    _log.warn('mouse movement test %s deprecated, converting to a MouseGesture', test)
-                self.__class__ = MouseGesture
-                self.__init__(MOUSE_GESTURE_TESTS[test], warn=warn)
-            elif test in TESTS:
-                self.function = TESTS[test]
-            else:
-                if warn:
-                    _log.warn('rule Test string argument not name of a test: %s', test)
-                self.function = TESTS['False']
-        elif isinstance(test, list) and all(isinstance(t, int) for t in test):
+            test = [test]
+        if isinstance(test, list) and all(isinstance(t, int) for t in test):
             if warn:
                 _log.warn('Test rules consisting of numbers are deprecated, converting to a TestBytes condition')
             self.__class__ = TestBytes
             self.__init__(test, warn=warn)
+        elif isinstance(test, list):
+            if test[0] in MOUSE_GESTURE_TESTS:
+                if warn:
+                    _log.warn('mouse movement test %s deprecated, converting to a MouseGesture', test)
+                self.__class__ = MouseGesture
+                self.__init__(MOUSE_GESTURE_TESTS[0][test], warn=warn)
+            elif test[0] in TESTS:
+                self.test = test[0]
+                self.function = TESTS[test[0]][0]
+                self.parameter = test[1] if len(test) > 1 else None
+            else:
+                if warn:
+                    _log.warn('rule Test name not name of a test: %s', test)
+                self.test = 'False'
+                self.function = TESTS['False'][0]
         else:
             if warn:
                 _log.warn('rule Test argument not valid %s', test)
@@ -756,13 +807,14 @@ class Test(Condition):
         return 'Test: ' + str(self.test)
 
     def evaluate(self, feature, notification, device, status, last_result):
-        return self.function(feature, notification.address, notification.data)
+        return self.function(feature, notification.address, notification.data, self.parameter)
 
     def data(self):
-        return {'Test': str(self.test)}
+        return {'Test': ([self.test, self.parameter] if self.parameter is not None else [self.test])}
 
 
 class TestBytes(Condition):
+
     def __init__(self, test, warn=True):
         self.test = test
         if (
@@ -832,6 +884,7 @@ class MouseGesture(Condition):
 
 
 class Active(Condition):
+
     def __init__(self, devID, warn=True):
         if not (isinstance(devID, str)):
             if warn:
@@ -851,6 +904,7 @@ class Active(Condition):
 
 
 class Action(RuleComponent):
+
     def __init__(self, *args):
         pass
 
@@ -925,14 +979,14 @@ class KeyPress(Action):
             (keycode, level) = self.keysym_to_keycode(k, modifiers)
             if keycode is None:
                 _log.warn('rule KeyPress key symbol not currently available %s', self)
-            elif self.needed(keycode, modifiers):
+            elif self.action != self.CLICK or self.needed(keycode, modifiers):  # only check needed when clicking
                 self.mods(level, modifiers, _KEY_PRESS)
                 simulate_key(keycode, _KEY_PRESS)
 
     def keyUp(self, keysyms, modifiers):
         for k in keysyms:
             (keycode, level) = self.keysym_to_keycode(k, modifiers)
-            if keycode and self.needed(keycode, modifiers):
+            if keycode and (self.action != self.CLICK or self.needed(keycode, modifiers)):  # only check needed when clicking
                 simulate_key(keycode, _KEY_RELEASE)
                 self.mods(level, modifiers, _KEY_RELEASE)
 
@@ -964,6 +1018,7 @@ class KeyPress(Action):
 
 
 class MouseScroll(Action):
+
     def __init__(self, amounts, warn=True):
         import numbers
         if len(amounts) == 1 and isinstance(amounts[0], list):
@@ -995,6 +1050,7 @@ class MouseScroll(Action):
 
 
 class MouseClick(Action):
+
     def __init__(self, args, warn=True):
         if len(args) == 1 and isinstance(args[0], list):
             args = args[0]
@@ -1029,6 +1085,7 @@ class MouseClick(Action):
 
 
 class Set(Action):
+
     def __init__(self, args, warn=True):
         if not (isinstance(args, list) and len(args) > 2):
             if warn:
@@ -1068,6 +1125,7 @@ class Set(Action):
 
 
 class Execute(Action):
+
     def __init__(self, args, warn=True):
         if isinstance(args, str):
             args = [args]
@@ -1092,6 +1150,39 @@ class Execute(Action):
         return {'Execute': self.args[:]}
 
 
+class Later(Action):
+
+    def __init__(self, args, warn=True):
+        self.delay = 0
+        self.rule = Rule([])
+        self.components = []
+        if not (isinstance(args, list)):
+            args = [args]
+        if not (isinstance(args, list) and len(args) >= 1):
+            if warn:
+                _log.warn('rule Later argument not list with minimum length 1: %s', args)
+        elif not (isinstance(args[0], int)) or not 0 < args[0] < 101:
+            if warn:
+                _log.warn('rule Later argument delay not integer between 1 and 100: %s', args)
+        else:
+            self.delay = args[0]
+            self.rule = Rule(args[1:], warn=warn)
+            self.components = self.rule.components
+
+    def __str__(self):
+        return 'Later: [' + str(self.delay) + ', ' + ', '.join(str(c) for c in self.components) + ']'
+
+    def evaluate(self, feature, notification, device, status, last_result):
+        if self.delay and self.rule:
+            GLib.timeout_add_seconds(self.delay, Rule.once, self.rule, feature, notification, device, status, last_result)
+        return None
+
+    def data(self):
+        data = [c.data() for c in self.components]
+        data.insert(0, self.delay)
+        return {'Later': data}
+
+
 COMPONENTS = {
     'Rule': Rule,
     'Not': Not,
@@ -1113,6 +1204,7 @@ COMPONENTS = {
     'MouseClick': MouseClick,
     'Set': Set,
     'Execute': Execute,
+    'Later': Later,
 }
 
 built_in_rules = Rule([])
@@ -1144,11 +1236,12 @@ keys_down = []
 g_keys_down = [0, 0, 0, 0]
 m_keys_down = 0
 mr_key_down = False
+thumb_wheel_displacement = 0
 
 
 # process a notification
 def process_notification(device, status, notification, feature):
-    global keys_down, g_keys_down, m_keys_down, mr_key_down, key_down, key_up
+    global keys_down, g_keys_down, m_keys_down, mr_key_down, key_down, key_up, thumb_wheel_displacement
     key_down, key_up = None, None
     # need to keep track of keys that are down to find a new key down
     if feature == _F.REPROG_CONTROLS_V4 and notification.address == 0x00:
@@ -1189,6 +1282,12 @@ def process_notification(device, status, notification, feature):
         if mr_key_down and not new_mr_key_down:
             key_up = _CONTROL['MR']
         mr_key_down = new_mr_key_down
+    # keep track of thumb wheel movment
+    elif feature == _F.THUMB_WHEEL and notification.address == 0x00:
+        if notification.data[4] <= 0x01:  # when wheel starts, zero out last movement
+            thumb_wheel_displacement = 0
+        thumb_wheel_displacement += signed(notification.data[0:2])
+
     GLib.idle_add(rules.evaluate, feature, notification, device, status, True)
 
 
