@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -19,17 +18,10 @@
 
 # Translation support for the Logitech receivers library
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import gettext as _gettext
 
-try:
-    unicode  # noqa: F821
-    _ = lambda x: _gettext.gettext(x).decode('UTF-8')
-    ngettext = lambda *x: _gettext.ngettext(*x).decode('UTF-8')
-except Exception:
-    _ = _gettext.gettext
-    ngettext = _gettext.ngettext
+_ = _gettext.gettext
+ngettext = _gettext.ngettext
 
 # A few common strings, not always accessible as such in the code.
 
